@@ -1,14 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2020-2024 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2020 -- leonerd@leonerd.org.uk
 
 package XS::Parse::Sublike::Builder;
-our $VERSION = '0.23_002'; # because package NAME VER doesn't work with _
 
-use v5.14;
+use strict;
 use warnings;
 
+our $VERSION = '0.01';
+
 =head1 NAME
 
 C<XS::Parse::Sublike::Builder> - build-time support for C<XS::Parse::Sublike>
@@ -39,25 +40,37 @@ distribution to be able to make use of C<XS::Parse::Sublike>.
 
 =cut
 
+my $XSParseSublike_h = do {
+   local $/;
+   readline DATA;
+};
+
 =head1 FUNCTIONS
 
 =cut
 
 =head2 write_XSParseSublike_h
 
-   XS::Parse::Sublike::Builder->write_XSParseSublike_h;
+   XS::Parse::Sublike::Builder->write_XSParseSublike_h
 
-This method no longer does anything I<since version 0.22>.
+Writes the F<XSParseSublike.h> file to the current working directory. To cause
+the compiler to actually find this file, see L</extra_compiler_flags>.
 
 =cut
 
 sub write_XSParseSublike_h
 {
+   shift;
+
+   open my $out, ">", "XSParseSublike.h" or
+      die "Cannot open XSParseSublike.h for writing - $!\n";
+
+   $out->print( $XSParseSublike_h );
 }
 
 =head2 extra_compiler_flags
 
-   @flags = XS::Parse::Sublike::Builder->extra_compiler_flags;
+   @flags = XS::Parse::Sublike::Builder->extra_compiler_flags
 
 Returns a list of extra flags that the build scripts should add to the
 compiler invocation. This enables the C compiler to find the
@@ -68,16 +81,12 @@ F<XSParseSublike.h> file.
 sub extra_compiler_flags
 {
    shift;
-
-   require File::ShareDir;
-   require File::Spec;
-   require XS::Parse::Sublike;
-   return "-I" . File::Spec->catdir( File::ShareDir::module_dir( "XS::Parse::Sublike" ), "include" );
+   return "-I.";
 }
 
 =head2 extend_module_build
 
-   XS::Parse::Sublike::Builder->extend_module_build( $build );
+   XS::Parse::Sublike::Builder->extend_module_build( $build )
 
 A convenient shortcut for performing all the tasks necessary to make a
 L<Module::Build>-based distribution use the helper.
@@ -89,6 +98,11 @@ sub extend_module_build
    my $self = shift;
    my ( $build ) = @_;
 
+   eval { $self->write_XSParseSublike_h } or do {
+      warn $@;
+      return;
+   };
+
    # preserve existing flags
    my @flags = @{ $build->extra_compiler_flags };
    push @flags, $self->extra_compiler_flags;
@@ -103,3 +117,31 @@ Paul Evans <leonerd@leonerd.org.uk>
 =cut
 
 0x55AA;
+
+__DATA__
+#ifndef __XS_PARSE_SUBLIKE_H__
+#define __XS_PARSE_SUBLIKE_H__
+
+struct XSParseSublikeHooks {
+  void (*post_blockstart)(pTHX);
+  OP * (*pre_blockend)   (pTHX_ OP *body);
+  void (*post_newcv)     (pTHX_ CV *cv);
+};
+
+#define xs_parse_sublike(hooks, op_ptr)  S_xs_parse_sublike(aTHX_ hooks, op_ptr)
+static int S_xs_parse_sublike(pTHX_ struct XSParseSublikeHooks *hooks, OP **op_ptr)
+{
+  SV *sv = get_sv("XS::Parse::Sublike::PARSE", 0);
+  if(!sv)
+    croak("Cannot find $XS::Parse::Sublike::PARSE - is it loaded?");
+
+  int (*func)(pTHX_ struct XSParseSublikeHooks *hooks, OP **op_ptr)
+    = INT2PTR(int (*)(pTHX_ struct XSParseSublikeHooks *, OP**), SvUV(sv));
+
+  return (*func)(aTHX_ hooks, op_ptr);
+}
+
+#define boot_xs_parse_sublike() \
+  load_module(PERL_LOADMOD_NOIMPORT, newSVpvs("XS::Parse::Sublike"), NULL, NULL)
+
+#endif
